@@ -313,6 +313,8 @@ void classRF(double *x, int *dimx, int *cl, int *ncl, int *cat, int *maxcat,
 					ntry++;
 				} while (anyEmpty && ntry <= 10);
 			}
+			/* If some classes are still empty, throw an error. */
+			if (anyEmpty) error("Some class has no data after 10 sampling attempts.");
 
             /* If need to keep indices of inbag data, do that here. */
             if (keepInbag) {
@@ -399,6 +401,22 @@ void classRF(double *x, int *dimx, int *cl, int *ncl, int *cat, int *maxcat,
 			R_CheckUserInterrupt();
 		}
 
+		/*  DO PROXIMITIES */
+		if (iprox) {
+            computeProximity(prox, oobprox, nodex, jin, oobpair, near);
+			/* proximity for test data */
+			if (*testdat) {
+                computeProximity(proxts, 0, nodexts, jin, oobpair, ntest);
+                /* Compute proximity between testset and training set. */
+				for (n = 0; n < ntest; ++n) {
+					for (k = 0; k < near; ++k) {
+						if (nodexts[n] == nodex[k])
+							proxts[n + ntest * (k+ntest)] += 1.0;
+					}
+				}
+			}
+		}
+
 		/*  DO VARIABLE IMPORTANCE  */
 		if (imp) {
 			nrightall = 0;
@@ -429,9 +447,11 @@ void classRF(double *x, int *dimx, int *cl, int *ncl, int *cat, int *maxcat,
 					/* Count how often correct predictions are made with
 					   the modified data. */
 					for (n = 0; n < nsample; n++) {
+						/* Restore the original data for that variable. */
+						x[m + n*mdim] = tx[n];
 						if (jin[n] == 0) {
 							if (jvr[n] == cl[n]) {
-								nrightimp[cl[n] - 1]++;
++								nrightimp[cl[n] - 1]++;
 								nrightimpall++;
 							}
 							if (localImp && jvr[n] != jtr[n]) {
@@ -442,8 +462,6 @@ void classRF(double *x, int *dimx, int *cl, int *ncl, int *cat, int *maxcat,
 								}
 							}
 						}
-						/* Restore the original data for that variable. */
-						x[m + n*mdim] = tx[n];
 					}
 					/* Accumulate decrease in proportions of correct
 					   predictions. */
@@ -468,21 +486,6 @@ void classRF(double *x, int *dimx, int *cl, int *ncl, int *cat, int *maxcat,
 			}
 		}
 
-		/*  DO PROXIMITIES */
-		if (iprox) {
-            computeProximity(prox, oobprox, nodex, jin, oobpair, near);
-			/* proximity for test data */
-			if (*testdat) {
-                computeProximity(proxts, 0, nodexts, jin, oobpair, ntest);
-                /* Compute proximity between testset and training set. */
-				for (n = 0; n < ntest; ++n) {
-					for (k = 0; k < near; ++k) {
-						if (nodexts[n] == nodex[k])
-							proxts[n + ntest * (k+ntest)] += 1.0;
-					}
-				}
-			}
-		}
 		R_CheckUserInterrupt();
 #ifdef WIN32
 		R_ProcessEvents();
@@ -584,11 +587,12 @@ void classForest(int *mdim, int *ntest, int *nclass, int *maxcat,
 			if (crit > cmax) {
 				jet[n] = j + 1;
 				cmax = crit;
+				ntie = 1;
 			}
 			/* Break ties at random: */
 			if (crit == cmax) {
+				if (unif_rand() < 1.0 / ntie) jet[n] = j + 1;
 				ntie++;
-				if (unif_rand() > 1.0 / ntie) jet[n] = j + 1;
 			}
 		}
     }
@@ -629,22 +633,23 @@ void oob(int nsample, int nclass, int *jin, int *cl, int *jtr,int *jerr,
 	    smaxtr = 0.0;
 	    ntie = 1;
 	    for (j = 0; j < nclass; ++j) {
-		qq = (((double) counttr[j + n*nclass]) / out[n]) / cutoff[j];
-		if (j+1 != cl[n]) smax = (qq > smax) ? qq : smax;
-		/* if vote / cutoff is larger than current max, re-set max and
-		   change predicted class to the current class */
-		if (qq > smaxtr) {
-		    smaxtr = qq;
-		    jest[n] = j+1;
-		}
-		/* break tie at random */
-		if (qq == smaxtr) {
-		    ntie++;
-		    if (unif_rand() > 1.0 / ntie) {
-			smaxtr = qq;
-			jest[n] = j+1;
-		    }
-		}
+	    	qq = (((double) counttr[j + n*nclass]) / out[n]) / cutoff[j];
+	    	if (j+1 != cl[n]) smax = (qq > smax) ? qq : smax;
+	    	/* if vote / cutoff is larger than current max, re-set max and
+		   	   change predicted class to the current class */
+	    	if (qq > smaxtr) {
+	    		smaxtr = qq;
+	    		jest[n] = j+1;
+	    		ntie = 1;
+	    	}
+	    	/* break tie at random */
+	    	if (qq == smaxtr) {
+	    		if (unif_rand() < 1.0 / ntie) {
+	    			smaxtr = qq;
+	    			jest[n] = j+1;
+	    		}
+	    		ntie++;
+	    	}
 	    }
 	    if (jest[n] != cl[n]) {
 		errtr[cl[n]] += 1.0;
@@ -675,14 +680,15 @@ void TestSetError(double *countts, int *jts, int *clts, int *jet, int ntest,
 			if (crit > cmax) {
 				jet[n] = j+1;
 				cmax = crit;
+				ntie = 1;
 			}
 			/*  Break ties at random: */
 			if (crit == cmax) {
-				ntie++;
-				if (unif_rand() > 1.0 / ntie) {
+				if (unif_rand() < 1.0 / ntie) {
 					jet[n] = j+1;
 					cmax = crit;
 				}
+				ntie++;
 			}
 		}
     }
